/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPGRAB_TYPES_H_
#define _NNTPGRAB_TYPES_H_

#include <sys/types.h>
#include <time.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* These declarations are actually from GLib, but to easy the development
 * of external frontends, they have been duplicated here to avoid linkage bloat */
#ifndef FALSE
#define FALSE   (0)
#endif

#ifndef TRUE
#define TRUE    (!FALSE)
#endif

typedef int                 ngboolean;
#ifdef WIN32
typedef long long           ngint64;
typedef unsigned long long  nguint64;
#else
typedef int64_t             ngint64;
typedef __uint64_t          nguint64;
#endif

typedef struct _NGList NGList;
typedef void NGRegex;

struct _NGList
{
  void *data;
  NGList *next;
  NGList *prev;
};

#define NG_CALLBACK(f)                   ((NGCallback) (f))
typedef void  (*NGCallback)              (void);

/* End of GLib declarations */

typedef enum {
    SERVER_PRIORITY_HIGH,
    SERVER_PRIORITY_NORMAL,
    SERVER_PRIORITY_LOW
} ServerPriority;

typedef struct _config_server {
    char servername[128];
    char hostname[128];
    int port;
    char username[64];
    char password[64];
    int max_threads;
    ServerPriority priority;
    ngboolean use_ssl;
    ngboolean enabled;
} ConfigServer;

typedef struct _config_opts {
    char download_directory[256];
    char temp_directory[256];
    ngboolean enable_intelligent_par2_downloading;
    ngboolean enable_par2_repair;
    char auto_import_directory[256];
    ngboolean enable_auto_import;
    ngboolean move_file_after_auto_import;
    ngboolean enable_auto_unpack;
    ngboolean enable_bandwidth_shaping;
    int max_bandwidth;
    ngboolean enable_webserver;
    int webserver_port;
    ngboolean enable_logger;
    ngboolean auto_remove_files;
} ConfigOpts;

typedef struct _config_group_info {
    char newsgroup[256];
    ngboolean is_subscribed;
    ngboolean is_moderated;
    int num_articles;
    int start_article;
    int end_article;
    time_t last_checked;
} ConfigGroupInfo;

typedef struct _nntpgrab_part {
    char message_id[256];
    int size;
    int part_num;
} NNTPGrabPart;

typedef struct {
    char name[128];
    char version[64];
    char author[128];
    char url[256];
    char description[4096];

    ngboolean is_loaded;
    ngboolean is_persistent;
} NNTPGrabPluginInfo;

typedef enum {
    CONFIG_CHANGED_NOTHING,
    CONFIG_CHANGED_SERVER_ADDED,
    CONFIG_CHANGED_SERVER_DELETED,
    CONFIG_CHANGED_OPTS_CHANGED
} ConfigChangedFlag;

typedef enum {
    SCHEDULAR_STATE_RUNNING,
    SCHEDULAR_STATE_STOPPING,
    SCHEDULAR_STATE_STOPPED
} NGSchedularState;

typedef enum _nntp_disconnect_type {
    DISCONNECT_NORMAL,
    DISCONNECT_NO_SUCH_HOST,
    DISCONNECT_CONNECTION_REFUSED,
    DISCONNECT_TOO_MANY_CONNECTIONS,
    DISCONNECT_CONNECT_TIMEOUT,
    DISCONNECT_READ_ERROR,
    DISCONNECT_READ_TIMEOUT,
    DISCONNECT_WRITE_ERROR,
    DISCONNECT_IDLE_TIMEOUT,
    DISCONNECT_INVALID_MSG,
    DISCONNECT_LOGIN_FAILURE,
    DISCONNECT_ERROR_SSL_INITIALISE,
    DISCONNECT_UNEXPECTED
} NNTPDisconnectType;

typedef enum _nntp_file_type {
    NNTP_FILE_TYPE_UNKNOWN,
    NNTP_FILE_TYPE_RAR,
    NNTP_FILE_TYPE_PAR,
    NNTP_FILE_TYPE_PAR2,
    NNTP_FILE_TYPE_SFV,
    NNTP_FILE_TYPE_NFO,
    NNTP_FILE_TYPE_NZB,
    NNTP_FILE_TYPE_OTHER
} NNTPFileType;

typedef enum {
    TASK_STATE_WAITING_FOR_DOWNLOAD,
    TASK_STATE_DOWNLOADING,
    TASK_STATE_WAITING_FOR_DECODE,
    TASK_STATE_DECODING,
    TASK_STATE_FINISHED_COMPLETE,
    TASK_STATE_FINISHED_INCOMPLETE,
    TASK_STATE_FINISHED_NO_PARTS_AVAIL,
    TASK_STATE_SKIPPED
} NGTaskState;

typedef enum _par2_file_state {
    PAR2_FILE_STATE_MISSING,
    PAR2_FILE_STATE_FOUND,
    PAR2_FILE_STATE_DAMAGED,
    PAR2_FILE_STATE_NO_NEW_BLOCKS_FOUND
} PAR2FileState;

typedef enum {
    NG_LOG_LEVEL_ALL = 0,
    NG_LOG_LEVEL_INFO,
    NG_LOG_LEVEL_WARNING,
    NG_LOG_LEVEL_ERROR,
    NG_LOG_LEVEL_FATAL,
    NG_LOG_LEVEL_DEBUG,
} NGLogLevel;

typedef void (*ForeachCollectionFunc) (const char *collection_name, const char *poster, nguint64 total_size, nguint64 total_size_remaining, int position, void *data);
typedef void (*ForeachFileFunc) (const char *collection_name, const char *subject, const char *poster, ngint64 stamp, nguint64 file_size, nguint64 file_size_remaining, int position, int num_parts_total, int num_parts_downloaded, int num_parts_failed, NGTaskState status, const char *filename, void *data);
typedef void (*ForeachGroupFunc) (const char *collection_name, const char *subject, const char *group, void *data);

#ifdef  __cplusplus
}
#endif

#endif /* _NNTPGRAB_TYPES_H_ */
