/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef _MSC_VER
#include "config.h.win32"
#else
#include "config.h"
#endif

#include <glib.h>
#include "nntpgrab_types.h"
#include "collections.h"
#include "collection_alloc.h"

NNTPFile *
file_new(void)
{
    NNTPFile *file = g_slice_new0(NNTPFile);
    file_ref(file);
    return file;
}

void
file_ref(NNTPFile *file)
{
    g_atomic_int_inc(&file->refcount);
//    g_print("file_ref: refcount = %i for subject %s\n", file->refcount, file->subject);
}

void
file_unref(NNTPFile *file)
{
    GList *groups;
    GList *parts;

    if (!g_atomic_int_dec_and_test(&file->refcount)) {
//        g_print("file_unref: refcount = %i for subject %s\n", file->refcount, file->subject);
        return;
    }

//    g_print("file_unref: refcount = %i for subject %s\n", file->refcount, file->subject);
//    g_print("releasing file\n");

    groups = file->groups;
    while (groups) {
        g_free(groups->data);
        groups = g_list_next(groups);
    }

    g_list_free(file->groups);

    parts = file->parts;
    while (parts) {
        g_slice_free(NNTPPart, parts->data);
        parts = g_list_next(parts);
    }

    g_list_free(file->parts);

    g_slice_free(NNTPFile, file);
}

NNTPCollection *
collection_new(void)
{
    NNTPCollection *collection = g_slice_new0(NNTPCollection);
    g_static_mutex_init(&collection->mutex);
    collection_ref(collection);
    return collection;
}

void
collection_ref(NNTPCollection *collection)
{
    g_atomic_int_inc(&collection->refcount);
//    g_print("collection_ref: refcount = %i for collection %s\n", collection->refcount, collection->collection_name);
}

void
collection_unref(NNTPCollection *collection)
{
    GList *files;

    if (!g_atomic_int_dec_and_test(&collection->refcount)) {
//        g_print("collection_unref: refcount = %i for collection %s\n", collection->refcount, collection->collection_name);
        return;
    }

//    g_print("collection_unref: refcount = %i for collection %s\n", collection->refcount, collection->collection_name);
//    g_print("releasing collection\n");

    files = collection->files;
    while (files) {
        NNTPFile *file = (NNTPFile *) files->data;

        file_unref(file);

        files = g_list_next(files);
    }

    g_list_free(collection->files);
    g_list_free(collection->files_to_download);

    g_slice_free(NNTPCollection, collection);
}
